//
// Copyright (c) 1999, Bernhard Tschirren
// All rights reserved.
//
// Redistribution and use in source and binary forms, with or without
// modification, are permitted provided that the following conditions
// are met:
//
// 1. Redistributions of the source code must retain the above copyright
//    notice, this list of conditions and the following disclaimer.
//
// 2. Any redistribution solely in binary form must conspicuously
//    reproduce the following disclaimer in documentation provided with
//    the binary redistribution.
//
// THIS SOFTWARE IS PROVIDED ``AS IS'', WITHOUT ANY WARRANTIES, EXPRESS
// OR IMPLIED, INCLUDING, BUT NOT LIMITED TO, WARRANTIES OF
// MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.  LICENSOR SHALL
// NOT BE LIABLE FOR ANY LOSS OR DAMAGES RESULTING FROM THE USE OF THIS
// SOFTWARE, EITHER ALONE OR IN COMBINATION WITH ANY OTHER SOFTWARE.
//

#ifndef _BT_COLOR_H_
#define _BT_COLOR_H_

#include "util/TTuple.h"

class Color : public Tuple4
  {
  public: Color()
            {
            Color::set(0, 0, 0, 1);
            }

  public: Color(Float r, Float g, Float b)
            {
            Color::set(r, g, b);
            }

  public: Color(Float r, Float g, Float b, Float a)
            {
            Color::set(r, g, b, a);
            }

  public: Color(const Color& other)
            {
            Color::set(other);
            }

  public: const Float* getPtr() const
            {
            return Tuple4::getPtr();
            }

  public: Float getR() const
            {
            return Tuple4::get(0);
            }

  public: Float getG() const
            {
            return Tuple4::get(1);
            }

  public: Float getB() const
            {
            return Tuple4::get(2);
            }

  public: Float getA() const
            {
            return Tuple4::get(3);
            }

  public: Color& set(const Color& other)
            {
            Tuple4::set(other);
            return *this;
            }

  public: Color& set(Float r, Float g, Float b)
            {
            Tuple4::set(0, FLOAT_CLAMP(r, 0,1));
            Tuple4::set(1, FLOAT_CLAMP(g, 0,1));
            Tuple4::set(2, FLOAT_CLAMP(b, 0,1));
            Tuple4::set(3, 1.0);

            return *this;
            }

  public: Color& set(Float r, Float g, Float b, Float a)
            {
            Tuple4::set(0, FLOAT_CLAMP(r, 0,1));
            Tuple4::set(1, FLOAT_CLAMP(g, 0,1));
            Tuple4::set(2, FLOAT_CLAMP(b, 0,1));
            Tuple4::set(3, FLOAT_CLAMP(a, 0,1));

            return *this;
            }

  TYPE_DATA // RTTI
  };

#endif /* _BT_COLOR_H_ */