//
// Copyright (c) 1999, Bernhard Tschirren
// All rights reserved.
//
// Redistribution and use in source and binary forms, with or without
// modification, are permitted provided that the following conditions
// are met:
//
// 1. Redistributions of the source code must retain the above copyright
//    notice, this list of conditions and the following disclaimer.
//
// 2. Any redistribution solely in binary form must conspicuously
//    reproduce the following disclaimer in documentation provided with
//    the binary redistribution.
//
// THIS SOFTWARE IS PROVIDED ``AS IS'', WITHOUT ANY WARRANTIES, EXPRESS
// OR IMPLIED, INCLUDING, BUT NOT LIMITED TO, WARRANTIES OF
// MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.  LICENSOR SHALL
// NOT BE LIABLE FOR ANY LOSS OR DAMAGES RESULTING FROM THE USE OF THIS
// SOFTWARE, EITHER ALONE OR IN COMBINATION WITH ANY OTHER SOFTWARE.
//

#ifndef _BT_GEOMETRY_H_
#define _BT_GEOMETRY_H_

#include "util/BaseClass.h"
#include "util/TArray.h"
#include "core/Surface.h"
#include "core/Material.h"
#include "core/Point.h"
#include "core/Vector.h"

class Geometry : public BaseClass
  {
  public: typedef struct
            {
            Float x, y, z;
            Float padding;
            } Coord;

  public: typedef struct
            {
            Float i, j, k;
            Float padding;
            } Normal;

  public: typedef TArray<Int>     IndexArray;
  public: typedef TArray<Coord>   CoordArray;
  public: typedef TArray<Normal>  NormalArray;
  public: typedef TArray<Surface> SurfaceArray;

  private: IndexArray   _indices;
  private: CoordArray   _coords;
  private: NormalArray  _normals;
  private: SurfaceArray _surfaces;
  private: Material*    _material;

  public: Geometry()
            : _indices(),
              _coords(),
              _normals(),
              _surfaces(),
              _material(NULL)
            {
            }

  public: const IndexArray& getIndices() const
            {
            return _indices;
            }

  public: const CoordArray& getCoords() const
            {
            return _coords;
            }

  public: const NormalArray& getNormals() const
            {
            return _normals;
            }

  public: const SurfaceArray& getSurfaces() const
            {
            return _surfaces;
            }

  public: Material* getMaterial() const
            {
            return _material;
            }

  public: Int addIndex(Int index)
            {
            return _indices.add(index);
            };

  public: Int addCoord(Float x, Float y, Float z)
            {
            Coord coord = { x, y, z, 0 };
            return _coords.add(coord);
            };

  public: Int addNormal(Float i, Float j, Float k)
            {
            Normal normal = { i, j, k, 0 };
            return _normals.add(normal);
            };

  public: Int addSurface(const Surface& surface)
            {
            return _surfaces.add(surface);
            }

  public: Void setMaterial(Material* material)
            {
            _material = material;
            }

  public: Void addVertex(const Point&  coord,
                         const Vector& normal)
            {
            addCoord( coord.getX(),  coord.getY(),  coord.getZ());
            addNormal(normal.getI(), normal.getJ(), normal.getK());
            }

  public: Void addTriangle(Int i1, Int i2, Int i3)
            {
            addIndex(i1);
            addIndex(i2);
            addIndex(i3);
            }

  TYPE_DATA // RTTI
  };

#endif /* _BT_GEOMETRY_H_ */