//
// Copyright (c) 1999, Bernhard Tschirren
// All rights reserved.
//
// Redistribution and use in source and binary forms, with or without
// modification, are permitted provided that the following conditions
// are met:
//
// 1. Redistributions of the source code must retain the above copyright
//    notice, this list of conditions and the following disclaimer.
//
// 2. Any redistribution solely in binary form must conspicuously
//    reproduce the following disclaimer in documentation provided with
//    the binary redistribution.
//
// THIS SOFTWARE IS PROVIDED ``AS IS'', WITHOUT ANY WARRANTIES, EXPRESS
// OR IMPLIED, INCLUDING, BUT NOT LIMITED TO, WARRANTIES OF
// MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.  LICENSOR SHALL
// NOT BE LIABLE FOR ANY LOSS OR DAMAGES RESULTING FROM THE USE OF THIS
// SOFTWARE, EITHER ALONE OR IN COMBINATION WITH ANY OTHER SOFTWARE.
//

#ifndef _BT_MATERIAL_H_
#define _BT_MATERIAL_H_

#include "util/BaseClass.h"
#include "util/TArray.h"
#include "core/Color.h"
#include "core/Texture.h"

class Material : public BaseClass
  {
  private: Color _ambient;
  private: Color _diffuse;
  private: Color _specular;
  private: Color _emission;
  private: Float _shininess;

  public: Material()
            : _ambient ((Float)0.2, (Float)0.2, (Float)0.2, (Float)1.0),
              _diffuse ((Float)0.8, (Float)0.8, (Float)0.8, (Float)1.0),
              _specular((Float)0.0, (Float)0.0, (Float)0.0, (Float)1.0),
              _emission((Float)0.0, (Float)0.0, (Float)0.0, (Float)1.0),
              _shininess(0)
            {
            }

  public: const Color& getAmbient() const
            {
            return _ambient;
            }

  public: const Color& getDiffuse() const
            {
            return _diffuse;
            }

  public: const Color& getSpecular() const
            {
            return _specular;
            }

  public: const Color& getEmission() const
            {
            return _emission;
            }

  public: Float getShininess() const
            {
            return _shininess;
            }

  public: Void setAmbient(const Color& ambient)
            {
            _ambient = ambient;
            }

  public: Void setDiffuse(const Color& diffuse)
            {
            _diffuse   = diffuse;
            }

  public: Void setSpecular(const Color& specular)
            {
            _specular  = specular;
            }

  public: Void setShininess(Float shininess)
            {
            _shininess = shininess;
            }

  TYPE_DATA // RTTI
  };

#endif /* _BT_MATERIAL_H_ */