//
// Copyright (c) 1999, Bernhard Tschirren
// All rights reserved.
//
// Redistribution and use in source and binary forms, with or without
// modification, are permitted provided that the following conditions
// are met:
//
// 1. Redistributions of the source code must retain the above copyright
//    notice, this list of conditions and the following disclaimer.
//
// 2. Any redistribution solely in binary form must conspicuously
//    reproduce the following disclaimer in documentation provided with
//    the binary redistribution.
//
// THIS SOFTWARE IS PROVIDED ``AS IS'', WITHOUT ANY WARRANTIES, EXPRESS
// OR IMPLIED, INCLUDING, BUT NOT LIMITED TO, WARRANTIES OF
// MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.  LICENSOR SHALL
// NOT BE LIABLE FOR ANY LOSS OR DAMAGES RESULTING FROM THE USE OF THIS
// SOFTWARE, EITHER ALONE OR IN COMBINATION WITH ANY OTHER SOFTWARE.
//

#ifndef _BT_MATRIX_H_
#define _BT_MATRIX_H_

#include "util/BaseClass.h"
#include "core/Point.h"
#include "core/Vector.h"

class Matrix : public BaseClass
  {
  private: Float _elements[4*4];

  public: Matrix()
            {
            _elements[ 0] = 1.0; _elements[ 1] = 0.0; _elements[ 2] = 0.0; _elements[ 3] = 0.0;
            _elements[ 4] = 0.0; _elements[ 5] = 1.0; _elements[ 6] = 0.0; _elements[ 7] = 0.0;
            _elements[ 8] = 0.0; _elements[ 9] = 0.0; _elements[10] = 1.0; _elements[11] = 0.0;
            _elements[12] = 0.0; _elements[13] = 0.0; _elements[14] = 0.0; _elements[15] = 1.0;
            }

  public: Float* getPtr()
            {
            return _elements;
            }

  public: Float get(Int row, Int col) const
            {
            return _elements[(col<<2)+row];
            }

  public: Void set(Int row, Int col, Float value)
            {
            _elements[(col<<2)+row] = value;
            }

  public: Point multiply(const Point& point) const
            {            
            Float rx, ry, rz;
            Float px, py, pz;

            px = point.getX();
            py = point.getY();
            pz = point.getZ();

            rx = _elements[0]*px + _elements[4]*py + _elements[ 8]*pz + _elements[12];
            ry = _elements[1]*px + _elements[5]*py + _elements[ 9]*pz + _elements[13];
            rz = _elements[2]*px + _elements[6]*py + _elements[10]*pz + _elements[14];

            return Point(rx, ry, rz);
            }

  public: Vector multiply(const Vector& vector) const
            {
            Float rx, ry, rz;
            Float px, py, pz;

            px = vector.getI();
            py = vector.getJ();
            pz = vector.getK();

            rx = _elements[0]*px + _elements[4]*py + _elements[ 8]*pz;
            ry = _elements[1]*px + _elements[5]*py + _elements[ 9]*pz;
            rz = _elements[2]*px + _elements[6]*py + _elements[10]*pz;

            return Vector(rx, ry, rz);
            }

  //public: Void multiply(const Matrix& matrix, Matrix* result) const
  //          {
  //          for (Int i = 0; i < 4; i++)
  //            {
  //            Float ai0 = get(i,0);
  //            Float ai1 = get(i,1);
  //            Float ai2 = get(i,2);
  //            Float ai3 = get(i,3);
  //
  //            result->set(i,0, ai0*matrix.get(0,0) + ai1*matrix.get(1,0) + ai2*matrix.get(2,0) + ai3*matrix.get(3,0));
  //            result->set(i,1, ai0*matrix.get(0,1) + ai1*matrix.get(1,1) + ai2*matrix.get(2,1) + ai3*matrix.get(3,1));
  //            result->set(i,2, ai0*matrix.get(0,2) + ai1*matrix.get(1,2) + ai2*matrix.get(2,2) + ai3*matrix.get(3,2));
  //            result->set(i,3, ai0*matrix.get(0,3) + ai1*matrix.get(1,3) + ai2*matrix.get(2,3) + ai3*matrix.get(3,3));
  //            }
  //          }

  TYPE_DATA // RTTI
  };

#endif /* _BT_MATRIX_H_ */