//
// Copyright (c) 1999, Bernhard Tschirren
// All rights reserved.
//
// Redistribution and use in source and binary forms, with or without
// modification, are permitted provided that the following conditions
// are met:
//
// 1. Redistributions of the source code must retain the above copyright
//    notice, this list of conditions and the following disclaimer.
//
// 2. Any redistribution solely in binary form must conspicuously
//    reproduce the following disclaimer in documentation provided with
//    the binary redistribution.
//
// THIS SOFTWARE IS PROVIDED ``AS IS'', WITHOUT ANY WARRANTIES, EXPRESS
// OR IMPLIED, INCLUDING, BUT NOT LIMITED TO, WARRANTIES OF
// MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.  LICENSOR SHALL
// NOT BE LIABLE FOR ANY LOSS OR DAMAGES RESULTING FROM THE USE OF THIS
// SOFTWARE, EITHER ALONE OR IN COMBINATION WITH ANY OTHER SOFTWARE.
//

#ifndef _BT_PLANE_H_
#define _BT_PLANE_H_

#include "util/BaseClass.h"
#include "core/Vector.h"
#include "core/Point.h"

class Plane : public BaseClass
  {
  private: Vector _normal;
  private: Float  _distance;

  public: Plane()
            : _normal(),
              _distance(0)
            {
            }

  public: Plane(const Vector& normal, Float distance)
            : _normal(normal),
              _distance(distance)
            {
            }

  public: Plane(const Point& p1, const Point& p2, const Point& p3)
            {
            set(p1, p2, p3);
            }

  public: Plane(const Plane& other)
            {
            set(other);
            }

  public: const Vector& getNormal() const
            {
            return _normal;
            }

  public: Float getDistance() const
            {
            return _distance;
            }

  public: Plane& set(const Plane& other)
            {
            _normal   = other._normal;
            _distance = other._distance;

            return *this;
            }

  public: Plane& set(const Point& p1, const Point& p2, const Point& p3)
            {
            Vector p1p2(p2 - p1);
            Vector p1p3(p3 - p1);
            
            ASSERT(!FLOAT_IS_ZERO(p1p2.length()));
            ASSERT(!FLOAT_IS_ZERO(p1p3.length()));  

            _normal   = p1p2.crossProduct(p1p3).normalise();
            _distance = p1.dotProduct(_normal);
            
            return *this;
            }

  public: Float distance(const Point& point) const
            {
            return point.dotProduct(_normal) - _distance;
            }

  TYPE_DATA // RTTI
  };

#endif /* _BT_PLANE_H_ */