//
// Copyright (c) 1999, Bernhard Tschirren
// All rights reserved.
//
// Redistribution and use in source and binary forms, with or without
// modification, are permitted provided that the following conditions
// are met:
//
// 1. Redistributions of the source code must retain the above copyright
//    notice, this list of conditions and the following disclaimer.
//
// 2. Any redistribution solely in binary form must conspicuously
//    reproduce the following disclaimer in documentation provided with
//    the binary redistribution.
//
// THIS SOFTWARE IS PROVIDED ``AS IS'', WITHOUT ANY WARRANTIES, EXPRESS
// OR IMPLIED, INCLUDING, BUT NOT LIMITED TO, WARRANTIES OF
// MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.  LICENSOR SHALL
// NOT BE LIABLE FOR ANY LOSS OR DAMAGES RESULTING FROM THE USE OF THIS
// SOFTWARE, EITHER ALONE OR IN COMBINATION WITH ANY OTHER SOFTWARE.
//

#ifndef _BT_POINT_H_
#define _BT_POINT_H_

#include "util/TTuple.h"
#include "core/Vector.h"

class Point : public Tuple3
  {
  public: Point()
            {
            Point::set(0, 0, 0);
            }

  public: Point(Float x, Float y, Float z)
            {
            Point::set(x, y, z);
            }

  public: Point(const Point& other)
            {
            Point::set(other);
            }

  public: Float getX() const
            {
            return Tuple3::get(0);
            }

  public: Float& getX()
            {
            return Tuple3::getRef(0);
            }

  public: Float getY() const
            {
            return Tuple3::get(1);
            }

  public: Float& getY()
            {
            return Tuple3::getRef(1);
            }

  public: Float getZ() const
            {
            return Tuple3::get(2);
            }

  public: Float& getZ()
            {
            return Tuple3::getRef(2);
            }

  public: Point& set(Float x, Float y, Float z)
            {
            Tuple3::set(0, x);
            Tuple3::set(1, y);
            Tuple3::set(2, z);
            return *this;
            }

  public: Point& set(const Point& other)
            {
            Tuple3::set(other);
            return *this;
            }

  public: Boolean equals(const Point& other) const
            {
            return Tuple3::equals(other);
            }

  public: Boolean isZero() const
            {
            return Tuple3::isZero();
            }

  public: Float dotProduct(const Vector& other) const
            {
            return Tuple3::dotProduct(other);
            }

  public: Point& interpolate(const Point& other, Float time)
            {
            Tuple3::interpolate(other, time);
            return *this;
            }

  public: const Point& operator =(const Point& other)
            {
            set(other);
            return *this;
            }

  public: Boolean operator ==(const Point& other) const
            {
            return equals(other);
            }

  public: Boolean operator !=(const Point& other) const
            {
            return !equals(other);
            }

  public: friend Point operator +(const Point& p, const Vector& v)
            {
            return Point(p.getX() + v.getI(),
                         p.getY() + v.getJ(),
                         p.getZ() + v.getK());
            }

  public: Point& operator +=(const Vector& v)
            {
            getX() += v.getI();
            getY() += v.getJ();
            getZ() += v.getK();

            return *this;
            }

  public: friend Point operator -(const Point& p, const Vector& v)
            {
            return Point(p.getX() - v.getI(),
                         p.getY() - v.getJ(),
                         p.getZ() - v.getK());
            }

  public: Point& operator -=(const Vector& v)
            {
            getX() -= v.getI();
            getY() -= v.getJ();
            getZ() -= v.getK();

            return *this;
            }

  public: friend Vector operator -(const Point& p1, const Point& p2)
            {
            return Vector(p1.getX() - p2.getX(),
                          p1.getY() - p2.getY(),
                          p1.getZ() - p2.getZ());
            }

  public: friend Point operator -(const Point& p)
            {
            return Point(-p.getX(), -p.getY(), -p.getZ());
            }

  TYPE_DATA // RTTI
  };

#endif /* _BT_POINT_H_ */