//
// Copyright (c) 1999, Bernhard Tschirren
// All rights reserved.
//
// Redistribution and use in source and binary forms, with or without
// modification, are permitted provided that the following conditions
// are met:
//
// 1. Redistributions of the source code must retain the above copyright
//    notice, this list of conditions and the following disclaimer.
//
// 2. Any redistribution solely in binary form must conspicuously
//    reproduce the following disclaimer in documentation provided with
//    the binary redistribution.
//
// THIS SOFTWARE IS PROVIDED ``AS IS'', WITHOUT ANY WARRANTIES, EXPRESS
// OR IMPLIED, INCLUDING, BUT NOT LIMITED TO, WARRANTIES OF
// MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.  LICENSOR SHALL
// NOT BE LIABLE FOR ANY LOSS OR DAMAGES RESULTING FROM THE USE OF THIS
// SOFTWARE, EITHER ALONE OR IN COMBINATION WITH ANY OTHER SOFTWARE.
//

#ifndef _BT_RENDERNODE_H_
#define _BT_RENDERNODE_H_

#include "util/Types.h"
#include "util/Graph.h"
#include "core/RenderProcess.h"
#include "core/RenderDevice.h"
#include "core/Point.h"
#include "core/Quaternion.h"
#include "core/Vector.h"

class RenderNode : public GraphNode
  {
  private: Point      _translation;
  private: Quaternion _rotation;
  private: Vector     _scale;

  protected: RenderNode()
               : _translation(),
                 _rotation(),
                 _scale(1, 1, 1)
               {
               }

  protected: RenderNode(const Point& translation, const Quaternion& rotation, const Vector& scale)
               : _translation(translation),
                 _rotation(rotation),
                 _scale(scale)
               {
               }

  public: Void setTranslation(const Point& translation)
            {
            _translation = translation;
            }

  public: const Point& getTranslation() const
            {
            return _translation;
            }

  public: Void setRotation(const Quaternion& rotation)
            {
            _rotation = rotation;
            }

  public: const Quaternion& getRotation() const
            {
            return _rotation;
            }

  public: Void setScale(const Vector& scale)
            {
            _scale = scale;
            }

  public: const Vector& getScale() const
            {
            return _scale;
            }

  public: RenderNode* getParent() const
            {
            if (getParents().isEmpty())
              return NULL;
            else
              return (RenderNode*)getParents()[0];
            }

  public: RenderNode* getChild() const
            {
            if (getChildren().isEmpty())
              return NULL;
            else
              return (RenderNode*)getChildren()[0];
            }

  public: virtual Boolean preRender(RenderProcess* process)
            {
            RenderDevice* device = process->getRenderDevice();

            device->pushMatrix();

            device->scale(getScale());
            device->translate(getTranslation());
            device->rotate(getRotation());

            return TRUE;
            }

  public: virtual Void postRender(RenderProcess* process)
            {
            RenderDevice* device = process->getRenderDevice();

            device->popMatrix();
            }

  TYPE_DATA // RTTI
  };

#endif /* _BT_RENDERNODE_H_ */