//
// Copyright (c) 1999, Bernhard Tschirren
// All rights reserved.
//
// Redistribution and use in source and binary forms, with or without
// modification, are permitted provided that the following conditions
// are met:
//
// 1. Redistributions of the source code must retain the above copyright
//    notice, this list of conditions and the following disclaimer.
//
// 2. Any redistribution solely in binary form must conspicuously
//    reproduce the following disclaimer in documentation provided with
//    the binary redistribution.
//
// THIS SOFTWARE IS PROVIDED ``AS IS'', WITHOUT ANY WARRANTIES, EXPRESS
// OR IMPLIED, INCLUDING, BUT NOT LIMITED TO, WARRANTIES OF
// MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.  LICENSOR SHALL
// NOT BE LIABLE FOR ANY LOSS OR DAMAGES RESULTING FROM THE USE OF THIS
// SOFTWARE, EITHER ALONE OR IN COMBINATION WITH ANY OTHER SOFTWARE.
//

#ifndef _BT_RENDERPROCESS_H_
#define _BT_RENDERPROCESS_H_

#include "util/BaseClass.h"
class RenderNode;
class RenderDevice;

class RenderProcess : public BaseClass
  {
  private: RenderDevice* _renderDevice;
  private: RenderNode*   _viewer;

  protected: RenderProcess()
               : _renderDevice(NULL),
                 _viewer(NULL)
               {
               }

  public: RenderDevice* getRenderDevice()
            {
            return _renderDevice;
            }

  public: Void setRenderDevice(RenderDevice* renderDevice)
            {
            _renderDevice = renderDevice;
            }

  public: RenderNode* getViewer()
            {
            return _viewer;
            }

  public: Void setViewer(RenderNode* viewer)
            {
            _viewer = viewer;
            }

  public: virtual Void render(RenderDevice* device) = 0;

  TYPE_DATA // RTTI
  };

class RenderException : public Exception
  {
  public: RenderException(const RenderProcess& process, const Char* format, ...)
            {
            va_list args;

            va_start(args, format);
            vsprintf(_message, format, args);
            va_end(args);
            };
  };

#endif /* _BT_RENDERPROCESS_H_ */