//
// Copyright (c) 1999, Bernhard Tschirren
// All rights reserved.
//
// Redistribution and use in source and binary forms, with or without
// modification, are permitted provided that the following conditions
// are met:
//
// 1. Redistributions of the source code must retain the above copyright
//    notice, this list of conditions and the following disclaimer.
//
// 2. Any redistribution solely in binary form must conspicuously
//    reproduce the following disclaimer in documentation provided with
//    the binary redistribution.
//
// THIS SOFTWARE IS PROVIDED ``AS IS'', WITHOUT ANY WARRANTIES, EXPRESS
// OR IMPLIED, INCLUDING, BUT NOT LIMITED TO, WARRANTIES OF
// MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.  LICENSOR SHALL
// NOT BE LIABLE FOR ANY LOSS OR DAMAGES RESULTING FROM THE USE OF THIS
// SOFTWARE, EITHER ALONE OR IN COMBINATION WITH ANY OTHER SOFTWARE.
//

#ifndef _BT_SCENE_H_
#define _BT_SCENE_H_

#include "util/Graph.h"
#include "util/TArray.h"
#include "core/Texture.h"
#include "core/Material.h"
#include "core/Geometry.h"
#include "core/RenderNode.h"
#include "parsers/Parser.h"

class Scene : public Graph
  {
  TArray<Texture*>  _textures;
  TArray<Material*> _materials;
  TArray<Geometry*> _geometry;

  public: Scene()
            : _textures(),
              _materials(),
              _geometry()
            {
            }

  public: Scene(const TArray<Parser::Symbol>& symbols)
            : _textures(),
              _materials(),
              _geometry()
            {           
            for (Int index = 0; index < symbols.getLength(); index++)
              {
              BaseClass*  object   = symbols[index].object;
              Texture*    texture  = PTR_CAST(Texture,    object);
              Material*   material = PTR_CAST(Material,   object);
              Geometry*   geometry = PTR_CAST(Geometry,   object);              
              RenderNode* node     = PTR_CAST(RenderNode, object);

              if (texture)  add(texture);
              if (material) add(material);
              if (geometry) add(geometry);
              if (node)     add(node);
              }
            }

  public: virtual ~Scene()
            {
            Int i;

            for (i = 0; i < _textures.getLength(); i++)
              delete _textures[i];

            for (i = 0; i < _materials.getLength(); i++)
              delete _materials[i];

            for (i = 0; i < _geometry.getLength(); i++)
              delete _geometry[i];
            }

  public: Int add(Texture* texture)
            {
            return _textures.addUnique(texture);
            }

  public: Int add(Material* material)
            {
            return _materials.addUnique(material);
            }

  public: Int add(Geometry* geometry)
            {
            const Geometry::SurfaceArray& surfaces = geometry->getSurfaces();

            // Add textures
            for (Int i = 0; i < surfaces.getLength(); i++)
              add(surfaces[i].getTexture());

            // Add material
            add(geometry->getMaterial());

            // Now, add the geometry
            return _geometry.addUnique(geometry);
            }

  public: Int add(RenderNode* node)
            {
            return Graph::add(node);
            }

  TYPE_DATA // RTTI
  };

#endif /* _BT_SCENE_H_ */