//
// Copyright (c) 1999, Bernhard Tschirren
// All rights reserved.
//
// Redistribution and use in source and binary forms, with or without
// modification, are permitted provided that the following conditions
// are met:
//
// 1. Redistributions of the source code must retain the above copyright
//    notice, this list of conditions and the following disclaimer.
//
// 2. Any redistribution solely in binary form must conspicuously
//    reproduce the following disclaimer in documentation provided with
//    the binary redistribution.
//
// THIS SOFTWARE IS PROVIDED ``AS IS'', WITHOUT ANY WARRANTIES, EXPRESS
// OR IMPLIED, INCLUDING, BUT NOT LIMITED TO, WARRANTIES OF
// MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.  LICENSOR SHALL
// NOT BE LIABLE FOR ANY LOSS OR DAMAGES RESULTING FROM THE USE OF THIS
// SOFTWARE, EITHER ALONE OR IN COMBINATION WITH ANY OTHER SOFTWARE.
//

#ifndef _BT_SURFACE_H_
#define _BT_SURFACE_H_

#include "util/BaseClass.h"
#include "util/TArray.h"
#include "core/Point.h"
#include "core/Vector.h"
#include "core/Quaternion.h"
#include "core/Texture.h"

class Surface : public BaseClass
  {
  public: typedef struct
            {
            Float s, t;
            Float padding1;
            Float padding2;
            } Coord;
  public: typedef TArray<Coord> CoordArray;

  private: Texture*   _texture;
  private: CoordArray _coordArray;
  private: Point      _translation;
  private: Quaternion _rotation;
  private: Vector     _scale;

  public: Surface()
            : _texture(NULL),
              _coordArray(),
              _translation(),
              _rotation(),
              _scale(1, 1, 1)
            {
            }

  public: Texture* getTexture() const
            {
            return _texture;
            }

  public: const CoordArray& getCoords() const
            {
            return _coordArray;
            }

  public: const Point& getTranslation() const
            {
            return _translation;
            }

  public: const Quaternion& getRotation() const
            {
            return _rotation;
            }

  public: const Vector& getScale() const
            {
            return _scale;
            }

  public: Void setTexture(Texture* texture)
            {
            _texture = texture;
            }
  
  public: Int addCoord(Float s, Float t)
            {
            Coord coord = { s, t, 0, 0 };
            return _coordArray.append(coord);
            }

  public: Void setTranslation(const Point& translation)
            {
            _translation = translation;
            }

  public: Void setRotation(const Quaternion& rotation)
            {
            _rotation = rotation;
            }

  public: Void setScale(const Vector& scale)
            {
            _scale = scale;
            }

  TYPE_DATA // RTTI
  };

#endif /* _BT_SURFACE_H_ */