//
// Copyright (c) 1999, Bernhard Tschirren
// All rights reserved.
//
// Redistribution and use in source and binary forms, with or without
// modification, are permitted provided that the following conditions
// are met:
//
// 1. Redistributions of the source code must retain the above copyright
//    notice, this list of conditions and the following disclaimer.
//
// 2. Any redistribution solely in binary form must conspicuously
//    reproduce the following disclaimer in documentation provided with
//    the binary redistribution.
//
// THIS SOFTWARE IS PROVIDED ``AS IS'', WITHOUT ANY WARRANTIES, EXPRESS
// OR IMPLIED, INCLUDING, BUT NOT LIMITED TO, WARRANTIES OF
// MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.  LICENSOR SHALL
// NOT BE LIABLE FOR ANY LOSS OR DAMAGES RESULTING FROM THE USE OF THIS
// SOFTWARE, EITHER ALONE OR IN COMBINATION WITH ANY OTHER SOFTWARE.
//

#ifndef _BT_TEXTURE_H_
#define _BT_TEXTURE_H_

#include "util/BaseClass.h"
#include "util/String.h"

class Texture : public BaseClass
  {
  public: typedef enum
            {
            REPLACE,
            MODULATE,
            DECAL,
            BLEND
            } Mode;

  private: String  _fileName;
  private: UInt    _handle;
  private: Boolean _wrapS;
  private: Boolean _wrapT;
  private: Boolean _mipMap;
  private: Mode    _mode;

  public: Texture()
            : _fileName(),
              _handle(0),
              _wrapS(TRUE),
              _wrapT(TRUE),
              _mode(MODULATE)
            {
            }

  public: Texture(const String& fileName)
            : _fileName(fileName),
              _handle(0),
              _wrapS(TRUE),
              _wrapT(TRUE),
              _mode(REPLACE)
            {
            }

  public: const String& getFileName() const
            {
            return _fileName;
            }

  public: UInt getHandle() const
            {
            return _handle;
            }

  public: Boolean getWrapS() const
            {
            return _wrapS;
            }

  public: Boolean getWrapT() const
            {
            return _wrapT;
            }

  public: Boolean getMipMap() const
            {
            return _mipMap;
            }

  public: Mode getMode() const
            {
            return _mode;
            }

  public: Void setFileName(const String& fileName)
            {
            _fileName = fileName;
            }

  public: Void setHandle(UInt handle)
            {
            _handle = handle;
            }

  public: Void setWrapS(Boolean wrap)
            {
            _wrapS = wrap;
            }

  public: Void setWrapT(Boolean wrap)
            {
            _wrapT = wrap;
            }

  public: Void setMipMap(Boolean mipMap)
            {
            _mipMap = mipMap;
            }

  public: Void setMode(Mode mode)
            {
            _mode = mode;
            }

  TYPE_DATA // RTTI
  };

#endif /* _BT_TEXTURE_H_ */