//
// Copyright (c) 1999, Bernhard Tschirren
// All rights reserved.
//
// Redistribution and use in source and binary forms, with or without
// modification, are permitted provided that the following conditions
// are met:
//
// 1. Redistributions of the source code must retain the above copyright
//    notice, this list of conditions and the following disclaimer.
//
// 2. Any redistribution solely in binary form must conspicuously
//    reproduce the following disclaimer in documentation provided with
//    the binary redistribution.
//
// THIS SOFTWARE IS PROVIDED ``AS IS'', WITHOUT ANY WARRANTIES, EXPRESS
// OR IMPLIED, INCLUDING, BUT NOT LIMITED TO, WARRANTIES OF
// MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.  LICENSOR SHALL
// NOT BE LIABLE FOR ANY LOSS OR DAMAGES RESULTING FROM THE USE OF THIS
// SOFTWARE, EITHER ALONE OR IN COMBINATION WITH ANY OTHER SOFTWARE.
//

#ifndef _BT_OPENGLRENDERDEVICE_H_
#define _BT_OPENGLRENDERDEVICE_H_

#include "core/RenderDevice.h"
#include "core/Texture.h"
#include "util/TArray.h"
#include "util/Library.h"

class OpenglRenderDevice : public RenderDevice
  {
  private: typedef enum
             {
             POINT_LIGHT,
             DIRECTIONAL_LIGHT,
             PLANAR_LIGHT,
             } LightType;

  private: typedef struct
             {
             LightType type;
             Color     ambient;
             Color     diffuse;
             Color     specular;
             Float     constantAttenuation;
             Float     linearAttenuation;
             Float     quadraticAttenuation;
             Float     exponent;
             Float     cutoff;
             Matrix    matrix;
             } Light;

  private: typedef TArray<Light>      LightArray;
  private: typedef TArray<LightArray> LightStack;

  private: Library*   _opengl;
  private: LightStack _lights;
  private: Int        _numTextureUnits;
  private: HDC        _hdc;
  private: HGLRC      _hglrc;

  public: OpenglRenderDevice(mxWindow* parent, Int x, Int y, Int w, Int h, const Char* label, Int style);
  public: virtual ~OpenglRenderDevice();

  public: virtual Void redraw();

  public: virtual Void setPerspective (Float left, Float right, Float bottom, Float top, Float zNear, Float zFar);
  public: virtual Void setOrthographic(Float left, Float right, Float bottom, Float top, Float zNear, Float zFar);

  public: virtual Void setRenderMode(RenderMode mode);

  public: virtual Void clear(Int mode);

  public: virtual Void beginScene();
  public: virtual Void endScene();
  
  public: virtual Int  beginObject(Int mode);
  public: virtual Void endObject(Int obj);
  public: virtual Void deleteObject(Int obj);
  
  public: virtual Void scale(const Vector& scale);
  public: virtual Void translate(const Point& translation);
  public: virtual Void rotate(const Quaternion& rotation);

  public: virtual Void   clearMatrix();
  public: virtual Void   setMatrix(const Matrix& matrix);
  public: virtual Matrix getMatrix();
  public: virtual Void   pushMatrix();
  public: virtual Void   popMatrix();

  public: virtual Void initLights         (const Boolean enabled,
                                           const Color& ambient);
  public: virtual Void addPointLight      (const Color& ambient,
                                           const Color& diffuse,
                                           const Color& specular,
                                           const Float  constantAttenuation,
                                           const Float  linearAttenuation,
                                           const Float  quadraticAttenuation);
  public: virtual Void addDirectionalLight(const Color& ambient,
                                           const Color& diffuse,
                                           const Color& specular,
                                           const Float  constantAttenuation,
                                           const Float  linearAttenuation,
                                           const Float  quadraticAttenuation,
                                           const Float  exponent,
                                           const Float  cutoff);
  public: virtual Void addPlanarLight     (const Color& ambient,
                                           const Color& diffuse,
                                           const Color& specular);  
  public: virtual Void clearLights();
  public: virtual Void pushLights();
  public: virtual Void popLights();

  public: virtual Void render(Int obj);
  public: virtual Void render(const Geometry& geometry);

  private: Void    newContext();
  private: Void    deleteContext();
  private: Void    setLight(Int depth, Int index);
  private: Boolean supportsExtension(const Char* extension);
  private: Void*   link(const Char* identifier, const Char* extension, Boolean optional);
  private: Void    uploadTexture(Texture* texture);

  TYPE_DATA // RTTI
  };

#endif /* _BT_OPENGLRENDERDEVICE_H_ */