//
// Copyright (c) 1999, Bernhard Tschirren
// All rights reserved.
//
// Redistribution and use in source and binary forms, with or without
// modification, are permitted provided that the following conditions
// are met:
//
// 1. Redistributions of the source code must retain the above copyright
//    notice, this list of conditions and the following disclaimer.
//
// 2. Any redistribution solely in binary form must conspicuously
//    reproduce the following disclaimer in documentation provided with
//    the binary redistribution.
//
// THIS SOFTWARE IS PROVIDED ``AS IS'', WITHOUT ANY WARRANTIES, EXPRESS
// OR IMPLIED, INCLUDING, BUT NOT LIMITED TO, WARRANTIES OF
// MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.  LICENSOR SHALL
// NOT BE LIABLE FOR ANY LOSS OR DAMAGES RESULTING FROM THE USE OF THIS
// SOFTWARE, EITHER ALONE OR IN COMBINATION WITH ANY OTHER SOFTWARE.
//

#ifndef _BT_LIGHTNODE_H_
#define _BT_LIGHTNODE_H_

#include "core/RenderNode.h"
#include "core/Color.h"

class LightNode : public RenderNode
  {
  protected: Color _ambient;
  protected: Color _diffuse;
  protected: Color _specular;
  protected: Float _constantAttenuation;
  protected: Float _linearAttenuation;
  protected: Float _quadraticAttenuation;

  protected: LightNode()
               : _ambient(Color(0, 0, 0, 1)),
                 _diffuse(Color(1, 1, 1, 1)),
                 _specular(Color(0, 0, 0, 1)),
                 _constantAttenuation(1),
                 _linearAttenuation(0),
                 _quadraticAttenuation(0)
               {
               }

  protected: LightNode(const Color& ambient,
                       const Color& diffuse,
                       const Color& specular,
                       const Float  constantAttenuation,
                       const Float  linearAttenuation,
                       const Float  quadraticAttenuation)
               : _ambient(ambient),
                 _diffuse(diffuse),
                 _specular(specular),
                 _constantAttenuation(constantAttenuation),
                 _linearAttenuation(linearAttenuation),
                 _quadraticAttenuation(quadraticAttenuation)
               {
               }

  public: Void setAmbient(const Color& ambient)
            {
            _ambient = ambient;
            }

  public: Void setDiffuse(const Color& diffuse)
            {
            _diffuse = diffuse;
            }

  public: Void setSpecular(const Color& specular)
            {
            _specular = specular;
            }

  public: Void setAttenuation(Float constant, Float linear, Float quadratic)
            {
            _constantAttenuation  = constant;
            _linearAttenuation    = constant;
            _quadraticAttenuation = constant;
            }

  TYPE_DATA // RTTI
  };

#endif /* _BT_LIGHTNODE_H_ */