//
// Copyright (c) 1999, Bernhard Tschirren
// All rights reserved.
//
// Redistribution and use in source and binary forms, with or without
// modification, are permitted provided that the following conditions
// are met:
//
// 1. Redistributions of the source code must retain the above copyright
//    notice, this list of conditions and the following disclaimer.
//
// 2. Any redistribution solely in binary form must conspicuously
//    reproduce the following disclaimer in documentation provided with
//    the binary redistribution.
//
// THIS SOFTWARE IS PROVIDED ``AS IS'', WITHOUT ANY WARRANTIES, EXPRESS
// OR IMPLIED, INCLUDING, BUT NOT LIMITED TO, WARRANTIES OF
// MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.  LICENSOR SHALL
// NOT BE LIABLE FOR ANY LOSS OR DAMAGES RESULTING FROM THE USE OF THIS
// SOFTWARE, EITHER ALONE OR IN COMBINATION WITH ANY OTHER SOFTWARE.
//

#include "parsers/Parser.h"
#include "core/Color.h"


DECLARE_OBJECT_PARSER(ParseColor)
DEFINE_OBJECT_PARSER (ParseColor)


BaseClass* ParseColor::parse(Parser& parser, BaseClass* owner)
  {
  Float  r;
  Float  g;
  Float  b;
  Float  a;
  Color* color;

  // Create Object
  color = new Color();
  
  // Parse Red Component
  parser.consume("r");
  parser.consume(Parser::tokEQUAL);
  r = parser.parseFloat();

  // Parse Green Component
  parser.consume("g");
  parser.consume(Parser::tokEQUAL);
  g = parser.parseFloat();

  // Parse Blue Component
  parser.consume("b");
  parser.consume(Parser::tokEQUAL);
  b = parser.parseFloat();

  // Parse Alpha component (optional)
  if (parser.curString() == "a")
    {
    parser.consume("a");
    parser.consume(Parser::tokEQUAL);
    a = parser.parseFloat();
    }
  else
    a = 1.0;

  // Set Properties
  color->set(r, g, b, a);

  return color;
  }