//
// Copyright (c) 1999, Bernhard Tschirren
// All rights reserved.
//
// Redistribution and use in source and binary forms, with or without
// modification, are permitted provided that the following conditions
// are met:
//
// 1. Redistributions of the source code must retain the above copyright
//    notice, this list of conditions and the following disclaimer.
//
// 2. Any redistribution solely in binary form must conspicuously
//    reproduce the following disclaimer in documentation provided with
//    the binary redistribution.
//
// THIS SOFTWARE IS PROVIDED ``AS IS'', WITHOUT ANY WARRANTIES, EXPRESS
// OR IMPLIED, INCLUDING, BUT NOT LIMITED TO, WARRANTIES OF
// MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.  LICENSOR SHALL
// NOT BE LIABLE FOR ANY LOSS OR DAMAGES RESULTING FROM THE USE OF THIS
// SOFTWARE, EITHER ALONE OR IN COMBINATION WITH ANY OTHER SOFTWARE.
//

#include "parsers/Parser.h"
#include "core/Geometry.h"


DECLARE_OBJECT_PARSER(ParseGeometry)
DEFINE_OBJECT_PARSER (ParseGeometry)


BaseClass* ParseGeometry::parse(Parser& parser, BaseClass* owner)
  {  
  Geometry*          geometry;
  Material*          material;
  TArray<Int>        indices;
  TArray<BaseClass*> coords;
  TArray<BaseClass*> normals;
  TArray<BaseClass*> surfaces;
  Int                i;

  // Create Object
  geometry = new Geometry();

  // Parse Indices
  parser.consume("indices");
  parser.consume(Parser::tokEQUAL);
  parser.parseIntArray(indices);

  // Parse Coords
  parser.consume("coords");
  parser.consume(Parser::tokEQUAL);
  parser.parseObjectArray(coords, geometry, STATIC_TYPE_INFO(Point));

  // Parse Normals (optional)
  if (parser.curString() == "normals")
    {
    parser.consume("normals");
    parser.consume(Parser::tokEQUAL);
    parser.parseObjectArray(normals, geometry, STATIC_TYPE_INFO(Vector));
    }

  // Parse Material (optional)
  if (parser.curString() == "material")
    {
    parser.consume("material");
    parser.consume(Parser::tokEQUAL);
    material = (Material*)parser.parseObject(geometry, STATIC_TYPE_INFO(Material));
    }
  else
    material = NULL;

  // Parse Surfaces
  if (parser.curString() == "surfaces")
    {
    parser.consume("surfaces");
    parser.consume(Parser::tokEQUAL);
    parser.parseObjectArray(surfaces, geometry, STATIC_TYPE_INFO(Surface));
    }

  for (i = 0; i < indices.getLength(); i++)
    geometry->addIndex(indices[i]);

  for (i = 0; i < coords.getLength(); i++)
    {
    const Point* coord = (const Point*)coords[i];
    geometry->addCoord(coord->getX(), coord->getY(), coord->getZ());
    delete coord;
    }

  for (i = 0; i < normals.getLength(); i++)
    {
    const Vector* normal = (const Vector*)normals[i];
    geometry->addNormal(normal->getI(), normal->getJ(), normal->getK());
    delete normal;
    }

  geometry->setMaterial(material);

  for (i = 0; i < surfaces.getLength(); i++)
    {
    const Surface* surface = (const Surface*)surfaces[i];
    geometry->addSurface(*surface);
    delete surface;
    }

  return geometry;
  }