//
// Copyright (c) 1999, Bernhard Tschirren
// All rights reserved.
//
// Redistribution and use in source and binary forms, with or without
// modification, are permitted provided that the following conditions
// are met:
//
// 1. Redistributions of the source code must retain the above copyright
//    notice, this list of conditions and the following disclaimer.
//
// 2. Any redistribution solely in binary form must conspicuously
//    reproduce the following disclaimer in documentation provided with
//    the binary redistribution.
//
// THIS SOFTWARE IS PROVIDED ``AS IS'', WITHOUT ANY WARRANTIES, EXPRESS
// OR IMPLIED, INCLUDING, BUT NOT LIMITED TO, WARRANTIES OF
// MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.  LICENSOR SHALL
// NOT BE LIABLE FOR ANY LOSS OR DAMAGES RESULTING FROM THE USE OF THIS
// SOFTWARE, EITHER ALONE OR IN COMBINATION WITH ANY OTHER SOFTWARE.
//

#include "parsers/Parser.h"


DECLARE_OBJECT_PARSER(ParseInclude)
DEFINE_OBJECT_PARSER (ParseInclude)


BaseClass* ParseInclude::parse(Parser& parser, BaseClass* owner)
  {
  // Consume object
  parser.consume("fileName");
  parser.consume(Parser::tokEQUAL);
  parser.expect(Parser::tokSTRING);

  // Create a new parser for included file
  FileStream stream(parser.curString(), "r");
  Parser     incParser(&stream);

  // Grab symbol lists
  TArray<Parser::Symbol>& curSymbols = parser.getSymbols();
  TArray<Parser::Symbol>& incSymbols = incParser.getSymbols();

  // Parse the included file
  incSymbols = curSymbols; // Copy current symbols to new parser
  incParser.parse();
  curSymbols = incSymbols; // Copy new symbols back to current parser

  // Consume file name
  parser.consume();

  return NULL;
  }