//
// Copyright (c) 1999, Bernhard Tschirren
// All rights reserved.
//
// Redistribution and use in source and binary forms, with or without
// modification, are permitted provided that the following conditions
// are met:
//
// 1. Redistributions of the source code must retain the above copyright
//    notice, this list of conditions and the following disclaimer.
//
// 2. Any redistribution solely in binary form must conspicuously
//    reproduce the following disclaimer in documentation provided with
//    the binary redistribution.
//
// THIS SOFTWARE IS PROVIDED ``AS IS'', WITHOUT ANY WARRANTIES, EXPRESS
// OR IMPLIED, INCLUDING, BUT NOT LIMITED TO, WARRANTIES OF
// MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.  LICENSOR SHALL
// NOT BE LIABLE FOR ANY LOSS OR DAMAGES RESULTING FROM THE USE OF THIS
// SOFTWARE, EITHER ALONE OR IN COMBINATION WITH ANY OTHER SOFTWARE.
//

#include "parsers/Parser.h"
#include "core/Material.h"


DECLARE_OBJECT_PARSER(ParseMaterial)
DEFINE_OBJECT_PARSER (ParseMaterial)


BaseClass* ParseMaterial::parse(Parser& parser, BaseClass* owner)
  {
  Material* material;
  Color*    ambient;
  Color*    diffuse;
  Color*    specular;
  Float     shininess;  

  // Create Object
  material = new Material();

  // Parse Ambient Color
  parser.consume("ambient");
  parser.consume(Parser::tokEQUAL);
  ambient = (Color*)parser.parseObject(material, STATIC_TYPE_INFO(Color));

  // Parse Diffuse Color
  parser.consume("diffuse");
  parser.consume(Parser::tokEQUAL);  
  diffuse = (Color*)parser.parseObject(material, STATIC_TYPE_INFO(Color));

  // Parse Specular Color
  parser.consume("specular");
  parser.consume(Parser::tokEQUAL);  
  specular = (Color*)parser.parseObject(material, STATIC_TYPE_INFO(Color));

  // Parse Shininess
  parser.consume("shininess");
  parser.consume(Parser::tokEQUAL);
  shininess = parser.parseFloat();

  material->setAmbient(*ambient);   delete(ambient);
  material->setDiffuse(*diffuse);   delete(diffuse);
  material->setSpecular(*specular); delete(specular);
  material->setShininess(shininess);
  
  return material;
  }