//
// Copyright (c) 1999, Bernhard Tschirren
// All rights reserved.
//
// Redistribution and use in source and binary forms, with or without
// modification, are permitted provided that the following conditions
// are met:
//
// 1. Redistributions of the source code must retain the above copyright
//    notice, this list of conditions and the following disclaimer.
//
// 2. Any redistribution solely in binary form must conspicuously
//    reproduce the following disclaimer in documentation provided with
//    the binary redistribution.
//
// THIS SOFTWARE IS PROVIDED ``AS IS'', WITHOUT ANY WARRANTIES, EXPRESS
// OR IMPLIED, INCLUDING, BUT NOT LIMITED TO, WARRANTIES OF
// MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.  LICENSOR SHALL
// NOT BE LIABLE FOR ANY LOSS OR DAMAGES RESULTING FROM THE USE OF THIS
// SOFTWARE, EITHER ALONE OR IN COMBINATION WITH ANY OTHER SOFTWARE.
//

#include "parsers/Parser.h"
#include "nodes/PointLightNode.h"
#include "core/Color.h"


DECLARE_OBJECT_PARSER(ParsePointLight)
DEFINE_OBJECT_PARSER (ParsePointLight)


BaseClass* ParsePointLight::parse(Parser& parser, BaseClass* owner)
  {
  PointLightNode* light;
  Point*          translation;
  Quaternion*     rotation;
  Color*          color;

  // Create Object
  light = new PointLightNode();

  // Parse Translation
  parser.consume("translation");
  parser.consume(Parser::tokEQUAL);
  translation = (Point*)parser.parseObject(light, STATIC_TYPE_INFO(Point));

  // Parse Rotation  
  parser.consume("rotation");
  parser.consume(Parser::tokEQUAL);
  rotation = (Quaternion*)parser.parseObject(light, STATIC_TYPE_INFO(Quaternion));

  // Parse Color
  parser.consume("diffuse");
  parser.consume(Parser::tokEQUAL);
  color = (Color*)parser.parseObject(light, STATIC_TYPE_INFO(Color));

  // Set Properties
  light->setTranslation(*translation); delete translation;
  light->setRotation(*rotation);       delete rotation;
  light->setDiffuse(*color);           delete color;
  light->setAmbient(Color(0.1,0.1,0.1));
  light->setSpecular(Color(0.1,0.1,0.1));

  return light;
  }