//
// Copyright (c) 1999, Bernhard Tschirren
// All rights reserved.
//
// Redistribution and use in source and binary forms, with or without
// modification, are permitted provided that the following conditions
// are met:
//
// 1. Redistributions of the source code must retain the above copyright
//    notice, this list of conditions and the following disclaimer.
//
// 2. Any redistribution solely in binary form must conspicuously
//    reproduce the following disclaimer in documentation provided with
//    the binary redistribution.
//
// THIS SOFTWARE IS PROVIDED ``AS IS'', WITHOUT ANY WARRANTIES, EXPRESS
// OR IMPLIED, INCLUDING, BUT NOT LIMITED TO, WARRANTIES OF
// MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.  LICENSOR SHALL
// NOT BE LIABLE FOR ANY LOSS OR DAMAGES RESULTING FROM THE USE OF THIS
// SOFTWARE, EITHER ALONE OR IN COMBINATION WITH ANY OTHER SOFTWARE.
//

#include "parsers/Parser.h"


DECLARE_OBJECT_PARSER(ParseReference)
DEFINE_OBJECT_PARSER (ParseReference)


BaseClass* ParseReference::parse(Parser& parser, BaseClass* owner)
  {
  Int        symbol;
  BaseClass* object;

  // Consume object
  parser.consume("obj");
  parser.consume(Parser::tokEQUAL);
  parser.expect(Parser::tokIDENTIFIER);

  // Find symbol
  symbol = parser.findSymbol(parser.curString());
  if (symbol < 0)
    throw ParserException(&parser, String().setFormatted("Object \"%s\" is not defined!", (const Char*)parser.curString()));

  // Make sure object has been defined
  object = parser.getSymbols()[symbol].object;
  if (object == NULL)
    throw ParserException(&parser, String().setFormatted("Object \"%s\" is declared, but not yet defined!", (const Char*)parser.curString()));

  // Consume object name
  parser.consume();

  return object;
  }