//
// Copyright (c) 1999, Bernhard Tschirren
// All rights reserved.
//
// Redistribution and use in source and binary forms, with or without
// modification, are permitted provided that the following conditions
// are met:
//
// 1. Redistributions of the source code must retain the above copyright
//    notice, this list of conditions and the following disclaimer.
//
// 2. Any redistribution solely in binary form must conspicuously
//    reproduce the following disclaimer in documentation provided with
//    the binary redistribution.
//
// THIS SOFTWARE IS PROVIDED ``AS IS'', WITHOUT ANY WARRANTIES, EXPRESS
// OR IMPLIED, INCLUDING, BUT NOT LIMITED TO, WARRANTIES OF
// MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.  LICENSOR SHALL
// NOT BE LIABLE FOR ANY LOSS OR DAMAGES RESULTING FROM THE USE OF THIS
// SOFTWARE, EITHER ALONE OR IN COMBINATION WITH ANY OTHER SOFTWARE.
//

#include "parsers/Parser.h"
#include "core/Surface.h"


DECLARE_OBJECT_PARSER(ParseSurface)
DEFINE_OBJECT_PARSER (ParseSurface)


BaseClass* ParseSurface::parse(Parser& parser, BaseClass* owner)
  {
  Surface*           surface;
  Texture*           texture;
  TArray<BaseClass*> texCoords;

  // Create Object
  surface = new Surface();

  // Parse texture
  parser.consume("texture");
  parser.consume(Parser::tokEQUAL);
  texture = (Texture*)parser.parseObject(surface, STATIC_TYPE_INFO(Texture));

  // Parse TexCoords
  parser.consume("texCoords");
  parser.consume(Parser::tokEQUAL);
  parser.parseObjectArray(texCoords, surface, STATIC_TYPE_INFO(Vector));

  // Set Properties
  surface->setTexture(texture);
  for (Int i = 0; i < texCoords.getLength(); i++)
    {
    const Vector* vector = (Vector*)texCoords[i];
    surface->addCoord(vector->getI(), vector->getJ());
    delete vector;
    }

  return surface;
  }