//
// Copyright (c) 1999, Bernhard Tschirren
// All rights reserved.
//
// Redistribution and use in source and binary forms, with or without
// modification, are permitted provided that the following conditions
// are met:
//
// 1. Redistributions of the source code must retain the above copyright
//    notice, this list of conditions and the following disclaimer.
//
// 2. Any redistribution solely in binary form must conspicuously
//    reproduce the following disclaimer in documentation provided with
//    the binary redistribution.
//
// THIS SOFTWARE IS PROVIDED ``AS IS'', WITHOUT ANY WARRANTIES, EXPRESS
// OR IMPLIED, INCLUDING, BUT NOT LIMITED TO, WARRANTIES OF
// MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.  LICENSOR SHALL
// NOT BE LIABLE FOR ANY LOSS OR DAMAGES RESULTING FROM THE USE OF THIS
// SOFTWARE, EITHER ALONE OR IN COMBINATION WITH ANY OTHER SOFTWARE.
//

#include "parsers/Parser.h"
#include "core/Texture.h"


DECLARE_OBJECT_PARSER(ParseTexture)
DEFINE_OBJECT_PARSER (ParseTexture)


BaseClass* ParseTexture::parse(Parser& parser, BaseClass* owner)
  {
  Texture* texture;

  // Create Object
  texture = new Texture();

  // Parse FileName
  parser.consume("fileName");
  parser.consume(Parser::tokEQUAL);
  texture->setFileName(parser.parseString());

  // Parse WrapS
  if (parser.curString() == "wrapS")
    {
    parser.consume("wrapS");
    parser.consume(Parser::tokEQUAL);
    texture->setWrapS(parser.parseBoolean());
    }

  // Parse WrapT
  if (parser.curString() == "wrapT")
    {
    parser.consume("wrapS");
    parser.consume(Parser::tokEQUAL);
    texture->setWrapT(parser.parseBoolean());
    }

  // Parse MipMap
  if (parser.curString() == "mipmap")
    {
    parser.consume("mipmap");
    parser.consume(Parser::tokEQUAL);
    texture->setMipMap(parser.parseBoolean());
    }

  // Parse Mode
  if (parser.curString() == "mode")
	{
	parser.consume("mode");
    parser.consume(Parser::tokEQUAL);
	if      (parser.curString() == "replace")
	  texture->setMode(Texture::REPLACE);
	else if (parser.curString() == "modulate")
	  texture->setMode(Texture::MODULATE);
	else if (parser.curString() == "decal")
	  texture->setMode(Texture::DECAL);
	else if (parser.curString() == "blend")
	  texture->setMode(Texture::BLEND);
	else
	  parser.consume("replace, modulate, decal or blend");
	parser.consume();
	}

  return texture;
  }