//
// Copyright (c) 1999, Bernhard Tschirren
// All rights reserved.
//
// Redistribution and use in source and binary forms, with or without
// modification, are permitted provided that the following conditions
// are met:
//
// 1. Redistributions of the source code must retain the above copyright
//    notice, this list of conditions and the following disclaimer.
//
// 2. Any redistribution solely in binary form must conspicuously
//    reproduce the following disclaimer in documentation provided with
//    the binary redistribution.
//
// THIS SOFTWARE IS PROVIDED ``AS IS'', WITHOUT ANY WARRANTIES, EXPRESS
// OR IMPLIED, INCLUDING, BUT NOT LIMITED TO, WARRANTIES OF
// MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.  LICENSOR SHALL
// NOT BE LIABLE FOR ANY LOSS OR DAMAGES RESULTING FROM THE USE OF THIS
// SOFTWARE, EITHER ALONE OR IN COMBINATION WITH ANY OTHER SOFTWARE.
//

#ifndef _BT_PARSER_H_
#define _BT_PARSER_H_

#include "util/Scanner.h"
#include "util/TArray.h"


class Parser : public Scanner
  {
  public: typedef struct
             {
             String     identifier;
             BaseClass* object;
             String     file;
             Int        line;
             } Symbol;

  private: TArray<Symbol> _symbols;

  public: Parser(FileStream* stream)
            : Scanner(stream),
              _symbols()
            {
            }

  public: Void       parse();
  public: BaseClass* find(const String& identifier, RTTItypeid type);

  public: Int        parseInt();
  public: Float      parseFloat();
  public: String     parseString();
  public: Boolean    parseBoolean();
  public: BaseClass* parseObject(BaseClass* owner, RTTItypeid type);
  public: BaseClass* parseParent(BaseClass* owner, const String& className);

  public: Void parseIntArray(TArray<Int>& array);
  public: Void parseFloatArray(TArray<Float>& array);
  public: Void parseObjectArray(TArray<BaseClass*>& array, BaseClass* owner, RTTItypeid type);

  public: Int             findSymbol(const String& identifier) const;
  public: Int             addSymbol(const String& identifier);
  public: Int             addSymbol(const Symbol& other);
  public: TArray<Symbol>& getSymbols() { return _symbols; }
  
  TYPE_DATA // RTTI
  };

class ParseObject : public BaseClass
  {
  public: virtual BaseClass* parse(Parser& parser, BaseClass* owner) = 0;
  TYPE_DATA // RTTI
  };

class ParserException : public ScannerException
  { public: ParserException(const Parser* parser, const String& message) : ScannerException(parser, message) {}; };

#define DECLARE_OBJECT_PARSER(T)                                          \
  class T : public ParseObject                                            \
    {                                                                     \
    public: T() {};                                                       \
    public: virtual BaseClass* parse(Parser& parser, BaseClass* owner);   \
    TYPE_DATA                                                             \
    };

#define DEFINE_OBJECT_PARSER(T)                                           \
  RTTI_DEF1_INST(T, #T, ParseObject)

#endif /* _BT_PARSER_H_ */