//
// Copyright (c) 1999, Bernhard Tschirren
// All rights reserved.
//
// Redistribution and use in source and binary forms, with or without
// modification, are permitted provided that the following conditions
// are met:
//
// 1. Redistributions of the source code must retain the above copyright
//    notice, this list of conditions and the following disclaimer.
//
// 2. Any redistribution solely in binary form must conspicuously
//    reproduce the following disclaimer in documentation provided with
//    the binary redistribution.
//
// THIS SOFTWARE IS PROVIDED ``AS IS'', WITHOUT ANY WARRANTIES, EXPRESS
// OR IMPLIED, INCLUDING, BUT NOT LIMITED TO, WARRANTIES OF
// MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.  LICENSOR SHALL
// NOT BE LIABLE FOR ANY LOSS OR DAMAGES RESULTING FROM THE USE OF THIS
// SOFTWARE, EITHER ALONE OR IN COMBINATION WITH ANY OTHER SOFTWARE.
//

#ifndef _BT_PORTALRENDERPROCESS_H_
#define _BT_PORTALRENDERPROCESS_H_

#include "core/RenderProcess.h"
#include "core/Frustum.h"
#include "util/TArray.h"

class PortalRenderProcess : public RenderProcess
  {
  private: Float            _fovY;
  private: Float            _aspect;
  private: Float            _zNear;
  private: Float            _zFar;
  private: TArray<Frustum*> _frustumStack;

  public: PortalRenderProcess(Float fovY, Float aspect, Float zNear, Float zFar)
            : RenderProcess(),
              _fovY(fovY),
              _aspect(aspect),
              _zNear(zNear),
              _zFar(zFar),
              _frustumStack()
            {
            }

  public: Void pushFrustum(Frustum* frustum)
            {
            _frustumStack.append(frustum);
            }

  public: Frustum* popFrustum()
            {            
            Int      depth   = _frustumStack.getLength();
			Frustum* frustum = getFrustum();

			if (depth > 0)
			  _frustumStack.setLength(depth-1);

            return frustum;
            }

  public: Frustum* getFrustum()
            {
			Int depth = _frustumStack.getLength();
			
			return (depth > 0) ? _frustumStack[depth-1] : NULL;
            }

  public: virtual Void render(RenderDevice* device);

  private: Void newFrustum();
  private: Void deleteFrustum();

  TYPE_DATA // RTTI
  };

#endif /* _BT_PORTALRENDERPROCESS_H_ */