//
// Copyright (c) 1999, Bernhard Tschirren
// All rights reserved.
//
// Redistribution and use in source and binary forms, with or without
// modification, are permitted provided that the following conditions
// are met:
//
// 1. Redistributions of the source code must retain the above copyright
//    notice, this list of conditions and the following disclaimer.
//
// 2. Any redistribution solely in binary form must conspicuously
//    reproduce the following disclaimer in documentation provided with
//    the binary redistribution.
//
// THIS SOFTWARE IS PROVIDED ``AS IS'', WITHOUT ANY WARRANTIES, EXPRESS
// OR IMPLIED, INCLUDING, BUT NOT LIMITED TO, WARRANTIES OF
// MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.  LICENSOR SHALL
// NOT BE LIABLE FOR ANY LOSS OR DAMAGES RESULTING FROM THE USE OF THIS
// SOFTWARE, EITHER ALONE OR IN COMBINATION WITH ANY OTHER SOFTWARE.
//

#ifndef _BT_BYTEORDER_H_
#define _BT_BYTEORDER_H_

#define ByteOrder_UNKNOWN         0
#define ByteOrder_BIG_ENDIAN      1
#define ByteOrder_LITTLE_ENDIAN   2

/* BEGIN CHEAT */
#define ByteOrder_LOCAL   ByteOrder_BIG_ENDIAN
/* END CHEAT */


#ifndef ByteOrder_LOCAL
#  define ByteOrder_LOCAL   ByteOrder_UNKNOWN
#endif

#if (ByteOrder_LOCAL == ByteOrder_LITTLE_ENDIAN)
#  define ByteOrder_ToNetwork8bits(value)     value
#  define ByteOrder_ToNetwork16bits(value)    value
#  define ByteOrder_ToNetwork32bits(value)    value
#  define ByteOrder_FromNetwork8bits(value)   value
#  define ByteOrder_FromNetwork16bits(value)  value
#  define ByteOrder_FromNetwork32bits(value)  value
#elif (ByteOrder_LOCAL == ByteOrder_BIG_ENDIAN)
/* BEGIN FIX-ME */
#  define ByteOrder_ToNetwork8bits(value)     value
#  define ByteOrder_ToNetwork16bits(value)    value
#  define ByteOrder_ToNetwork32bits(value)    value
#  define ByteOrder_FromNetwork8bits(value)   value
#  define ByteOrder_FromNetwork16bits(value)  value
#  define ByteOrder_FromNetwork32bits(value)  value
/* END FIX-ME */
#else
#  error  Unknown local byte-order (ByteOrder_LOCAL)
#endif

#define ByteOrder_ToNetworkBoolean(value)   ByteOrder_ToNetwork8bits(value)
#define ByteOrder_ToNetworkChar(value)      ByteOrder_ToNetwork8bits(value)
#define ByteOrder_ToNetworkUChar(value)     ByteOrder_ToNetwork8bits(value)
#define ByteOrder_ToNetworkByte(value)      ByteOrder_ToNetwork8bits(value)
#define ByteOrder_ToNetworkUByte(value)     ByteOrder_ToNetwork8bits(value)
#define ByteOrder_ToNetworkShort(value)     ByteOrder_ToNetwork16bits(value)
#define ByteOrder_ToNetworkUShort(value)    ByteOrder_ToNetwork16bits(value)
#define ByteOrder_ToNetworkLong(value)      ByteOrder_ToNetwork32bits(value)
#define ByteOrder_ToNetworkULong(value)     ByteOrder_ToNetwork32bits(value)
#define ByteOrder_ToNetworkInt(value)       ByteOrder_ToNetwork32bits(value)
#define ByteOrder_ToNetworkUInt(value)      ByteOrder_ToNetwork32bits(value)

#define ByteOrder_FromNetworkBoolean(value) ByteOrder_FromNetwork8bits(value)
#define ByteOrder_FromNetworkChar(value)    ByteOrder_FromNetwork8bits(value)
#define ByteOrder_FromNetworkUChar(value)   ByteOrder_FromNetwork8bits(value)
#define ByteOrder_FromNetworkByte(value)    ByteOrder_FromNetwork8bits(value)
#define ByteOrder_FromNetworkUByte(value)   ByteOrder_FromNetwork8bits(value)
#define ByteOrder_FromNetworkShort(value)   ByteOrder_FromNetwork16bits(value)
#define ByteOrder_FromNetworkUShort(value)  ByteOrder_FromNetwork16bits(value)
#define ByteOrder_FromNetworkLong(value)    ByteOrder_FromNetwork32bits(value)
#define ByteOrder_FromNetworkULong(value)   ByteOrder_FromNetwork32bits(value)
#define ByteOrder_FromNetworkInt(value)     ByteOrder_FromNetwork32bits(value)
#define ByteOrder_FromNetworkUInt(value)    ByteOrder_FromNetwork32bits(value)

#endif /* _BT_BYTEORDER_H_ */