//
// Copyright (c) 1999, Bernhard Tschirren
// All rights reserved.
//
// Redistribution and use in source and binary forms, with or without
// modification, are permitted provided that the following conditions
// are met:
//
// 1. Redistributions of the source code must retain the above copyright
//    notice, this list of conditions and the following disclaimer.
//
// 2. Any redistribution solely in binary form must conspicuously
//    reproduce the following disclaimer in documentation provided with
//    the binary redistribution.
//
// THIS SOFTWARE IS PROVIDED ``AS IS'', WITHOUT ANY WARRANTIES, EXPRESS
// OR IMPLIED, INCLUDING, BUT NOT LIMITED TO, WARRANTIES OF
// MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.  LICENSOR SHALL
// NOT BE LIABLE FOR ANY LOSS OR DAMAGES RESULTING FROM THE USE OF THIS
// SOFTWARE, EITHER ALONE OR IN COMBINATION WITH ANY OTHER SOFTWARE.
//

#ifndef _BT_EXCEPTIONS_H_
#define _BT_EXCEPTIONS_H_

#include "util/Types.h"

#if defined(_DEBUG)
#  define ASSERT_SRC(condition, srcName, srcLine) if (!(condition)) throw AssertionException("%s:%d: assertion failed: %s", srcName, srcLine, #condition)
#  define VERITY_SRC(condition, srcName, srcLine) ASSERT(condition, srcName, srcLine)
#else
#  define ASSERT_SRC(condition, srcName, srcLine)
#  define VERIFY_SRC(condition, srcName, srcLine) (condition)
#endif
#define ASSERT(condition) ASSERT_SRC(condition, __FILE__, __LINE__)
#define VERIFY(condition) VERIFY_SRC(condition, __FILE__, __LINE__)

#define ASSERT_ALWAYS_SRC(condition, srcName, srcLine) if (!(condition)) throw AssertionException("%s:%d: assertion failed: %s", srcName, srcLine, #condition)
#define ASSERT_ALWAYS(condition)                       ASSERT_ALWAYS_SRC(condition, __FILE__, __LINE__)


class Exception
  {
  protected: Char _message[1024];

  public: Exception()
            {
            strcpy(_message, "An unknown exception has occurred!");
            };

  public: Exception(const Char* format, ...)
            {
            va_list args;

            va_start(args, format);
            vsprintf(_message, format, args);
            va_end(args);
            };

  public: const Char* getMessage() const
            {
            return _message;
            };
  };


class AbstractException : public Exception
  {
  public: AbstractException(const Char* className, const Char* methodName)
            : Exception("Tried to call abstract method %s::%s()", className, methodName)
            {};
  };


class InvalidArgumentException : public Exception
  {
  public: InvalidArgumentException(const Char* format, ...)
            {
            va_list args;

            va_start(args, format);
            vsprintf(_message, format, args);
            va_end(args);
            };
  };

class AssertionException : public Exception
  {
  public: AssertionException(const Char* format, ...)
            {
            va_list args;

            va_start(args, format);
            vsprintf(_message, format, args);
            va_end(args);
            };
  };

#endif /* _BT_EXCEPTIONS_H_ */