//
// Copyright (c) 1999, Bernhard Tschirren
// All rights reserved.
//
// Redistribution and use in source and binary forms, with or without
// modification, are permitted provided that the following conditions
// are met:
//
// 1. Redistributions of the source code must retain the above copyright
//    notice, this list of conditions and the following disclaimer.
//
// 2. Any redistribution solely in binary form must conspicuously
//    reproduce the following disclaimer in documentation provided with
//    the binary redistribution.
//
// THIS SOFTWARE IS PROVIDED ``AS IS'', WITHOUT ANY WARRANTIES, EXPRESS
// OR IMPLIED, INCLUDING, BUT NOT LIMITED TO, WARRANTIES OF
// MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.  LICENSOR SHALL
// NOT BE LIABLE FOR ANY LOSS OR DAMAGES RESULTING FROM THE USE OF THIS
// SOFTWARE, EITHER ALONE OR IN COMBINATION WITH ANY OTHER SOFTWARE.
//

#include "util/FileStream.h"


FileStream::FileStream(FILE* handle)
  {
  ASSERT(handle != NULL);

  _handle   = handle;
  _fileName = "";
  _fileName.compact();
  }


FileStream::FileStream(const String& fileName, const String& fileMode)
  {
  _handle = fopen(fileName.getPtr(), fileMode.getPtr());
  if (_handle == NULL)
    throw StreamCreateException(String().setFormatted("Could not open '%s' (mode=%s)", fileName.getPtr(), fileMode.getPtr()));

  _fileName = fileName;
  _fileName.compact();
  }


FileStream::~FileStream()
  {
  ASSERT(_handle != NULL);

  fclose(_handle);
  _handle = NULL;
  }


Int FileStream::getLength()
  {
  Int pos;
  Int size;

  pos = ftell(_handle);
  fseek(_handle, 0, 2);
  size = ftell(_handle);
  fseek(_handle, pos, 0);

  return size;
  }


Void FileStream::setLength(Int length)
  {
  Int pos;

  pos = ftell(_handle);
  fseek(_handle, length, 0);
  fseek(_handle, pos, 0);
  }


Boolean FileStream::endOfStream()
  {
  Int ch;

  ch = getc(_handle);
  if (ch != EOF)
    ungetc(ch, _handle);

  return (ch == EOF);
  }


Void FileStream::read(Void* data, Int size)
  {
  Int numRead;

  numRead = fread(data, 1, size, _handle);
  if (numRead != size)
    throw StreamReadException(String().setFormatted("Could not read %d bytes from '%s' (numRead=%d)", size, _fileName.getPtr(), numRead));
  }


Void FileStream::write(const Void* data, Int size)
  {
  Int numWritten;

  numWritten = fwrite(data, 1, size, _handle);
  if (numWritten != size)
    throw StreamWriteException(String().setFormatted("Could not write %d bytes to '%s' (numWritten=%d)", size, _fileName.getPtr(), numWritten));
  }


Void FileStream::seek(Int offset, Int mode)
  {
  fseek(_handle, offset, mode);
  }


Void FileStream::flush()
  {
  fflush(_handle);
  }


Void FileStream::printf(const Char* format, ...)
  {
  va_list args;

  va_start(args, format);
  vprintf(format, args);
  va_end(args);
  }


Void FileStream::vprintf(const Char* format, va_list args)
  {
  vfprintf(_handle, format, args);
  }


RTTI_DEF1(FileStream, "FileStream", Stream)