//
// Copyright (c) 1999, Bernhard Tschirren
// All rights reserved.
//
// Redistribution and use in source and binary forms, with or without
// modification, are permitted provided that the following conditions
// are met:
//
// 1. Redistributions of the source code must retain the above copyright
//    notice, this list of conditions and the following disclaimer.
//
// 2. Any redistribution solely in binary form must conspicuously
//    reproduce the following disclaimer in documentation provided with
//    the binary redistribution.
//
// THIS SOFTWARE IS PROVIDED ``AS IS'', WITHOUT ANY WARRANTIES, EXPRESS
// OR IMPLIED, INCLUDING, BUT NOT LIMITED TO, WARRANTIES OF
// MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.  LICENSOR SHALL
// NOT BE LIABLE FOR ANY LOSS OR DAMAGES RESULTING FROM THE USE OF THIS
// SOFTWARE, EITHER ALONE OR IN COMBINATION WITH ANY OTHER SOFTWARE.
//

#ifndef _BT_FILESTREAM_H_
#define _BT_FILESTREAM_H_

#include "util/Stream.h"
#include <stdio.h>

class FileStream : public Stream
  {
  private: FILE*  _handle;
  private: String _fileName;

  public:  FileStream(FILE* handle);
  public:  FileStream(const String& fileName, const String& fileMode);
  public: ~FileStream();

  public: const FILE*   getHandle() const   { return _handle;   };
  public: const String& getFileName() const { return _fileName; };

  public: virtual Int     getLength();
  public: virtual Void    setLength(Int);
  public: virtual Boolean endOfStream();

  public: virtual Void    read(Void *data, Int size);
  public: virtual Void    write(const Void *data, Int size);
  public: virtual Void    seek(Int offset, Int mode=0);
  public: virtual Void    flush();

  public: virtual Void    printf(const Char* format, ...);
  public: virtual Void    vprintf(const Char* format, va_list args);

  TYPE_DATA // RTTI
  };

#endif /* _BT_FILESTREAM_H_ */