//
// Copyright (c) 1999, Bernhard Tschirren
// All rights reserved.
//
// Redistribution and use in source and binary forms, with or without
// modification, are permitted provided that the following conditions
// are met:
//
// 1. Redistributions of the source code must retain the above copyright
//    notice, this list of conditions and the following disclaimer.
//
// 2. Any redistribution solely in binary form must conspicuously
//    reproduce the following disclaimer in documentation provided with
//    the binary redistribution.
//
// THIS SOFTWARE IS PROVIDED ``AS IS'', WITHOUT ANY WARRANTIES, EXPRESS
// OR IMPLIED, INCLUDING, BUT NOT LIMITED TO, WARRANTIES OF
// MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.  LICENSOR SHALL
// NOT BE LIABLE FOR ANY LOSS OR DAMAGES RESULTING FROM THE USE OF THIS
// SOFTWARE, EITHER ALONE OR IN COMBINATION WITH ANY OTHER SOFTWARE.
//

#ifndef _BT_GLOBALS_H_
#define _BT_GLOBALS_H_

#include "util/TArray.h"
#include "util/String.h"
#include "util/FileStream.h"

class Globals
  {
  private: typedef struct
             {
             String name;
             Char   type;
             union
               {
               Boolean valueb;  // Boolean
               Int     valuei;  // Int
               Float   valuef;  // Float
               String* values;  // String
               };
             } Value;

  private: static TArray<Value> _values;

  public: static Boolean getValueb(const String& name, Boolean defaultValue=FALSE);
  public: static Void    setValueb(const String& name, Boolean value);

  public: static Int     getValuei(const String& name, Int defaultValue=0);
  public: static Void    setValuei(const String& name, Int value);

  public: static Float   getValuef(const String& name, Float defaultValue=0.0);
  public: static Void    setValuef(const String& name, Float value);

  public: static String  getValues(const String& name, const String& defaultValue="");
  public: static Void    setValues(const String& name, const String& value);

  public: static Void    load(FileStream& stream);
  public: static Void    save(FileStream& stream);

  private: static const Char* typeName(Char type);
  private: static Boolean     find(const String& name, Char type, Int* index);
  };

#endif /* _BT_GLOBALS_H_ */