//
// Copyright (c) 1999, Bernhard Tschirren
// All rights reserved.
//
// Redistribution and use in source and binary forms, with or without
// modification, are permitted provided that the following conditions
// are met:
//
// 1. Redistributions of the source code must retain the above copyright
//    notice, this list of conditions and the following disclaimer.
//
// 2. Any redistribution solely in binary form must conspicuously
//    reproduce the following disclaimer in documentation provided with
//    the binary redistribution.
//
// THIS SOFTWARE IS PROVIDED ``AS IS'', WITHOUT ANY WARRANTIES, EXPRESS
// OR IMPLIED, INCLUDING, BUT NOT LIMITED TO, WARRANTIES OF
// MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.  LICENSOR SHALL
// NOT BE LIABLE FOR ANY LOSS OR DAMAGES RESULTING FROM THE USE OF THIS
// SOFTWARE, EITHER ALONE OR IN COMBINATION WITH ANY OTHER SOFTWARE.
//

#include "util/Graph.h"


Void GraphDFS::execute(GraphNode* startNode)
  {
  ASSERT(startNode != NULL);

  _processed.clear();
  executeInternal(startNode);
  }


Void GraphDFS::executeInternal(GraphNode* node)
  {
  ASSERT(node != NULL);

  if (!_processed.contains(node))                                 // Dont process marked nodes
    {
    if (preProcess(node))                                         // Allow user to pre-process node
      {
      _processed.add(node);                                       // Mark node as processed
      
      const GraphNodeArray& children = node->getChildren();       // Get the list of children

      for (Int index = 0; index < children.getLength(); index++)  // For all children...
        executeInternal(children[index]);                         //   Recursively process

      postProcess(node);                                          // Allow user to post-process node
      }
    }
  }


RTTI_DEF1_INST(GraphNode, "GraphNode", BaseClass)
RTTI_DEF1_INST(Graph,     "Graph",     BaseClass)
RTTI_DEF1_INST(GraphDFS,  "GraphDFS",  BaseClass)