//
// Copyright (c) 1999, Bernhard Tschirren
// All rights reserved.
//
// Redistribution and use in source and binary forms, with or without
// modification, are permitted provided that the following conditions
// are met:
//
// 1. Redistributions of the source code must retain the above copyright
//    notice, this list of conditions and the following disclaimer.
//
// 2. Any redistribution solely in binary form must conspicuously
//    reproduce the following disclaimer in documentation provided with
//    the binary redistribution.
//
// THIS SOFTWARE IS PROVIDED ``AS IS'', WITHOUT ANY WARRANTIES, EXPRESS
// OR IMPLIED, INCLUDING, BUT NOT LIMITED TO, WARRANTIES OF
// MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.  LICENSOR SHALL
// NOT BE LIABLE FOR ANY LOSS OR DAMAGES RESULTING FROM THE USE OF THIS
// SOFTWARE, EITHER ALONE OR IN COMBINATION WITH ANY OTHER SOFTWARE.
//

#ifndef _BT_MEMORY_H_
#define _BT_MEMORY_H_

#include "util/Exceptions.h"

class Memory
  {
  private: static const Int GUARD;
  private: static const Int LEADING_GUARD_SIZE;
  private: static const Int TRAILING_GUARD_SIZE;
  private: static const Int TOTAL_GUARD_SIZE;

  private: static Int _maxMemoryUsed;
  private: static Int _memoryUsed;

  public:  static Int getPageSize();
  public:  static Int getVirtualAvailable();
  public:  static Int getPhysicalAvailable();
  public:  static Int getMemoryUsed()                    { return _memoryUsed;                  };
  public:  static Int getMaxMemoryUsed()                 { return _maxMemoryUsed;               };
  public:  static Int getMemorySize(const Void* pointer) { return *getSizePtr(pointer);         };
  public:  static Int getPageNumber(const Void* pointer) { return (Int)pointer / getPageSize(); };
  
  public:  static Void* allocate(Int size, const Char* srcName, Int srcLine);
  public:  static Void* reallocate(Void *pointer, Int size, const Char* srcName, Int srcLine);
  public:  static Void  free(Void* pointer, const Char* srcName, Int srcLine);

  private: static Int* getSizePtr(const Void* pointer)      { return (Int*)(&((Int *)pointer)[-2]);                   };
  private: static Int* getLowGuardPtr(const Void* pointer)  { return (Int*)(&((Int *)pointer)[-1]);                   };
  private: static Int* getHighGuardPtr(const Void* pointer) { return (Int*)(&((Byte*)pointer)[*getSizePtr(pointer)]); };

  public:  static Void checkGuards(const Void* pointer, const Char* srcName, Int srcLine) { ASSERT_SRC(*getLowGuardPtr(pointer) == GUARD, srcName, srcLine); ASSERT_SRC(*getHighGuardPtr(pointer) == GUARD, srcName, srcLine); };
  };

class MemoryException      : public Exception { };
class OutOfMemoryException : public MemoryException { public: OutOfMemoryException(const Char* format, ...); };

#endif /* _BT_MEMORY_H_ */