//
// Copyright (c) 1999, Bernhard Tschirren
// All rights reserved.
//
// Redistribution and use in source and binary forms, with or without
// modification, are permitted provided that the following conditions
// are met:
//
// 1. Redistributions of the source code must retain the above copyright
//    notice, this list of conditions and the following disclaimer.
//
// 2. Any redistribution solely in binary form must conspicuously
//    reproduce the following disclaimer in documentation provided with
//    the binary redistribution.
//
// THIS SOFTWARE IS PROVIDED ``AS IS'', WITHOUT ANY WARRANTIES, EXPRESS
// OR IMPLIED, INCLUDING, BUT NOT LIMITED TO, WARRANTIES OF
// MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.  LICENSOR SHALL
// NOT BE LIABLE FOR ANY LOSS OR DAMAGES RESULTING FROM THE USE OF THIS
// SOFTWARE, EITHER ALONE OR IN COMBINATION WITH ANY OTHER SOFTWARE.
//

#ifndef _BT_SCANNER_H_
#define _BT_SCANNER_H_

#include "util/BaseClass.h"
#include "util/FileStream.h"
#include "util/String.h"

class Scanner : public BaseClass
  {
  public: typedef enum
            {
            tokINVALID,      // Invalid character
            tokEOF,          // End Of File
            tokCOMMENT,      // # Comment
            tokIDENTIFIER,   // Identifier
            tokSTRING,       // "String"
            tokFLOAT,        // 12.203
            tokINTEGER,      // 12
            tokEQUAL,        // =
            tokPERIOD,       // .
            tokCOMMA,        // ,
            tokCOLON,        // :
            tokSEMICOLON,    // ;
            tokLEFTBRACE,    // {
            tokRIGHTBRACE,   // }
            tokLEFTBRACKET,  // [
            tokRIGHTBRACKET  // ]
            } Token;

  private: FileStream& _stream;
  private: Int         _line;
  private: Char        _char;
  private: String      _string;
  private: Token       _token;

  public: Scanner(FileStream* stream);

  public: Void expect(Token token);
  public: Void expect(Token token1, Token token2);
  public: Void consume();
  public: Void consume(Token token);
  public: Void consume(const Char* identifier);

  public: const String& curFile() const   { return _stream.getFileName(); };
  public: Int           curLine() const   { return _line;                 };
  public: Char          curChar() const   { return _char;                 };
  public: const String& curString() const { return _string;               };
  public: Token         curToken() const  { return _token;                };

  private: Token       scanIdentifier();
  private: Token       scanNumber();
  private: Token       scanString();
  private: Token       scanComment();  
  private: Char        scanChar();
  private: Token       scanToken();
  private: const Char* tokenName(Token token) const;

  TYPE_DATA // RTTI
  };

class ScannerException : public Exception
  { public: ScannerException(const Scanner* scanner, const String& message) : Exception("%s:%d: %s", scanner->curFile().getPtr(), scanner->curLine(), message.getPtr()) {}; };

#endif /* _BT_SCANNER_H_ */