//
// Copyright (c) 1999, Bernhard Tschirren
// All rights reserved.
//
// Redistribution and use in source and binary forms, with or without
// modification, are permitted provided that the following conditions
// are met:
//
// 1. Redistributions of the source code must retain the above copyright
//    notice, this list of conditions and the following disclaimer.
//
// 2. Any redistribution solely in binary form must conspicuously
//    reproduce the following disclaimer in documentation provided with
//    the binary redistribution.
//
// THIS SOFTWARE IS PROVIDED ``AS IS'', WITHOUT ANY WARRANTIES, EXPRESS
// OR IMPLIED, INCLUDING, BUT NOT LIMITED TO, WARRANTIES OF
// MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.  LICENSOR SHALL
// NOT BE LIABLE FOR ANY LOSS OR DAMAGES RESULTING FROM THE USE OF THIS
// SOFTWARE, EITHER ALONE OR IN COMBINATION WITH ANY OTHER SOFTWARE.
//

#include "util/Stream.h"
#include "util/ByteOrder.h"


Void Stream::readChar(Char* data)
  {
  read(data, sizeof(Char));
  *data = ByteOrder_FromNetworkChar(*data);
  }


Void Stream::readByte(Byte* data)
  {
  read(data, sizeof(Char));
  *data = ByteOrder_FromNetworkByte(*data);
  }


Void Stream::readShort(Short* data)
  {
  read(data, sizeof(Short));
  *data = ByteOrder_FromNetworkShort(*data);
  }


Void Stream::readLong(Long* data)
  {
  read(data, sizeof(Long));
  *data = ByteOrder_FromNetworkLong(*data);
  }


Void Stream::readString(String* data)
  {
  Long  length;
  Char* string;

  readLong(&length);

  data->setLength(length);
  string = data->getPtr();

  read(string, length*sizeof(Char));
  string[length] = '\0';
  }


Void Stream::writeChar(Char data)
  {
  data = ByteOrder_ToNetworkChar(data);
  write(&data, sizeof(Char));
  }


Void Stream::writeByte(Byte data)
  {
  data = ByteOrder_ToNetworkByte(data);
  write(&data, sizeof(Byte));
  }


Void Stream::writeShort(Short data)
  {
  data = ByteOrder_ToNetworkShort(data);
  write(&data, sizeof(Short));
  }


Void Stream::writeLong(Long data)
  {
  data = ByteOrder_ToNetworkLong(data);
  write(&data, sizeof(Long));
  }


Void Stream::writeString(const String& data)
  {
  Long        length;
  const Char* string;

  length = data.getLength();
  string = data.getPtr();

  writeLong(length);
  write(string, length);
  }


RTTI_DEF1(Stream, "Stream", BaseClass)