//
// Copyright (c) 1999, Bernhard Tschirren
// All rights reserved.
//
// Redistribution and use in source and binary forms, with or without
// modification, are permitted provided that the following conditions
// are met:
//
// 1. Redistributions of the source code must retain the above copyright
//    notice, this list of conditions and the following disclaimer.
//
// 2. Any redistribution solely in binary form must conspicuously
//    reproduce the following disclaimer in documentation provided with
//    the binary redistribution.
//
// THIS SOFTWARE IS PROVIDED ``AS IS'', WITHOUT ANY WARRANTIES, EXPRESS
// OR IMPLIED, INCLUDING, BUT NOT LIMITED TO, WARRANTIES OF
// MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.  LICENSOR SHALL
// NOT BE LIABLE FOR ANY LOSS OR DAMAGES RESULTING FROM THE USE OF THIS
// SOFTWARE, EITHER ALONE OR IN COMBINATION WITH ANY OTHER SOFTWARE.
//

#ifndef _BT_STREAM_H_
#define _BT_STREAM_H_

#include "util/BaseClass.h"
#include "util/String.h"

class Stream : public BaseClass
  {
  public: virtual Void read(Void* data, Int size) = 0;
  public: virtual Void readChar(Char* data);
  public: virtual Void readByte(Byte* data);
  public: virtual Void readShort(Short* data);
  public: virtual Void readLong(Long* data);
  public: virtual Void readString(String* data);

  public: virtual Void write(const Void* data, Int size) = 0;
  public: virtual Void writeChar(Char data);
  public: virtual Void writeByte(Byte data);
  public: virtual Void writeShort(Short data);
  public: virtual Void writeLong(Long data);
  public: virtual Void writeString(const String& data);

  public: virtual Boolean endOfStream() = 0;

  TYPE_DATA // RTTI
  };


class StreamException : public Exception
  { public: StreamException(const String& message) : Exception(message.getPtr()) {}; };

class StreamCreateException : public StreamException
  { public: StreamCreateException(const String& message) : StreamException(message) {}; };

class StreamReadException : public StreamException
  { public: StreamReadException(const String& message) : StreamException(message) {}; };

class StreamWriteException : public StreamException
  { public: StreamWriteException(const String& message) : StreamException(message) {}; };

class StreamNotSupported : public StreamException
  { public: StreamNotSupported(const String& message) : StreamException(message) {}; };

#endif /* _BT_STREAM_H_ */