//
// Copyright (c) 1999, Bernhard Tschirren
// All rights reserved.
//
// Redistribution and use in source and binary forms, with or without
// modification, are permitted provided that the following conditions
// are met:
//
// 1. Redistributions of the source code must retain the above copyright
//    notice, this list of conditions and the following disclaimer.
//
// 2. Any redistribution solely in binary form must conspicuously
//    reproduce the following disclaimer in documentation provided with
//    the binary redistribution.
//
// THIS SOFTWARE IS PROVIDED ``AS IS'', WITHOUT ANY WARRANTIES, EXPRESS
// OR IMPLIED, INCLUDING, BUT NOT LIMITED TO, WARRANTIES OF
// MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.  LICENSOR SHALL
// NOT BE LIABLE FOR ANY LOSS OR DAMAGES RESULTING FROM THE USE OF THIS
// SOFTWARE, EITHER ALONE OR IN COMBINATION WITH ANY OTHER SOFTWARE.
//

#ifndef _BT_TTUPLE_H_
#define _BT_TTUPLE_H_

#include "util/BaseClass.h"

template<int S> class TTuple : public BaseClass
  {
  private: Float _elements[S];

  protected: TTuple()
               {
               for (Int i = 0; i < S; i++)
                 _elements[i] = 0.0;
               }

  protected: const Float* getPtr() const
               {
               return _elements;
               };

  protected: Float get(Int index) const
               {
               return _elements[index];
               };

  protected: Float& getRef(Int index)
               {
               return _elements[index];
               };

  protected: Void set(Int index, Float element)
               {
               _elements[index] = element;
               };

  protected: Void set(const TTuple<S>& other)
               {
               for (Int i = 0; i < S; i++)
                 _elements[i] = other._elements[i];
               };

  protected: Boolean equals(const TTuple<S>& other) const
               {
               for (Int i = 0; i < S; i++)
                 {
                 if (!FLOAT_EQUALS(_elements[i], other._elements[i]))
                   return FALSE;
                 }
               return TRUE;
               };

  protected: Boolean isZero() const
               {
               for (Int i = 0; i < S; i++)
                 {
                 if (!FLOAT_EQUALS(_elements[i], 0))
                   return FALSE;
                 }
               return TRUE;
               };

  protected: Float lengthSquared() const
               {
               Float len = 0;

               for (Int i = 0; i < S; i++)
                  len += (_elements[i] * _elements[i]);

               return len;
               };

  protected: Float length() const
               {
               return (Float)sqrt(lengthSquared());
               };

  protected: Void scale(Float factor)
               {
               for (Int i = 0; i < S; i++)
                 _elements[i] *= factor;
               };

  protected: Void scale(const TTuple<S>& factor)
               {
               for (Int i = 0; i < S; i++)
                 _elements[i] *= factor._elements[i];
               };

  protected: Float dotProduct(const TTuple<S>& other) const
               {
               Float dp = 0;

               for (Int i = 0; i < S; i++)
                  dp += (_elements[i] * other._elements[i]);

               return dp;
               };

  protected: Void interpolate(const TTuple<S>& other, Float time)
               {
               time = FLOAT_CLAMP(time, 0, 1);

               for (Int i = 0; i < S; i++)
                  _elements[i] += ((other._elements[i] - _elements[i]) * time);
               };
  };

typedef TTuple<2> Tuple2;
typedef TTuple<3> Tuple3;
typedef TTuple<4> Tuple4;

#endif /* _BT_TTUPLE_H_ */