//
// Copyright (c) 1999, Bernhard Tschirren
// All rights reserved.
//
// Redistribution and use in source and binary forms, with or without
// modification, are permitted provided that the following conditions
// are met:
//
// 1. Redistributions of the source code must retain the above copyright
//    notice, this list of conditions and the following disclaimer.
//
// 2. Any redistribution solely in binary form must conspicuously
//    reproduce the following disclaimer in documentation provided with
//    the binary redistribution.
//
// THIS SOFTWARE IS PROVIDED ``AS IS'', WITHOUT ANY WARRANTIES, EXPRESS
// OR IMPLIED, INCLUDING, BUT NOT LIMITED TO, WARRANTIES OF
// MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.  LICENSOR SHALL
// NOT BE LIABLE FOR ANY LOSS OR DAMAGES RESULTING FROM THE USE OF THIS
// SOFTWARE, EITHER ALONE OR IN COMBINATION WITH ANY OTHER SOFTWARE.
//

#ifndef _BT_TYPES_H_
#define _BT_TYPES_H_

#include <stdio.h>
#include <stdlib.h>
#include <stdarg.h>
#include <errno.h>
#include <math.h>
#include <limits.h>

#if defined(_MSC_VER)
#  pragma warning (disable:4244)  /* disable bogus conversion warnings */
#  pragma warning (disable:4305)  /* VC++ 5.0 version of above warning */
#endif

#if defined(WIN32) || defined(_WIN32) || defined(_WIN32_) || defined(__WIN32__)
#  define  WIN32_LEAN_AND_MEAN
#  include <windows.h>
#  if !defined(_WIN32)
#    define _WIN32
#  endif
#else
#  include <unistd.h>
#  if !defined(_UNIX)
#    define _UNIX
#  endif
#endif

#if defined(DEBUG) || defined(_DEBUG) || defined(_DEBUG_) || defined(__DEBUG__)
#  if !defined(_DEBUG)
#    define _DEBUG
#  endif
#endif

#if !defined(M_PI)
#  define M_PI  3.14159265358979323846
#endif

#if defined(__cplusplus)
#  if !defined(TRUE) || !defined(FALSE)
#    define FALSE  false
#    define TRUE   true
#  endif
#else
# define bool int
# if !defined(TRUE) || !defined(FALSE)
#   define FALSE  (0)
#   define TRUE   (!FALSE)
# endif
#endif

#if defined(_MSC_VER)
#  define DLL_EXPORT  __declspec(dllexport)
#endif

typedef bool            Boolean;
typedef void            Void;
typedef char            Char;
typedef unsigned char   UChar;
typedef char            Byte;       /* ==  8-bits */
typedef unsigned char   UByte;      /* ==  8-bits */
typedef short           Short;      /* == 16-bits */
typedef unsigned short  UShort;     /* == 16-bits */
typedef long            Long;       /* == 32-bits */
typedef unsigned long   ULong;      /* == 32-bits */
typedef int             Int;        /* >= 32-bits */
typedef unsigned int    UInt;       /* >= 32-bits */
typedef float           Float;
typedef double          Double;

#define MAX_BYTE        (0x7F        )
#define MIN_BYTE        (-MAX_BYTE-1 )

#define MAX_SHORT       (0x7FFF      )
#define MIN_SHORT       (-MAX_SHORT-1)

#define MAX_LONG        (0x7FFFFFFF  )
#define MIN_LONG        (-MAX_LONG-1 )

#define MAX_INT         INT_MAX
#define MIN_INT         INT_MIN

#ifndef MIN
#  define MIN(a,b)  (((a) < (b)) ? a : b)
#endif

#ifndef MAX
#  define MAX(a,b)  (((a) > (b)) ? a : b)
#endif

#define FLOAT_ACCURACY        ((Float)0.00001)
#define FLOAT_EQUALS(a, b)    (fabs((Float)(a)-(Float)(b)) < FLOAT_ACCURACY)
#define FLOAT_IS_ZERO(x)      (FLOAT_EQUALS(x, 0.0))
#define FLOAT_CLAMP(x, a,b)   (MIN(MAX((Float)(x), (Float)(a)), (Float)(b)))

#endif /* _BT_TYPES_H_ */